﻿##############################
# $Id: htpasswd.p,v 1.22 2004/01/12 13:08:12 egr Exp $
# Класс для работы с .htpasswd
# Идея Pavel Titov (pavel@titov.pp.ru)
# Я сильно усовершенствовал реализацию - вместо таблиц используется хэш
#
# Использование:
# $oPass[^htpasswd::init[/path/to/.htpasswd]]
#
# Добавление (изменение пароля) пользователя с передачей
# незашифрованного пароля
# ^oPass.setpw[$sUserName;$sPasswd]
#
# Добавление (изменение пароля) пользователя с передачей
# хэша пользовательского пароля, полученного с помощью math:crypt
# ^oPass.setpwCrypted[$sUserName;$sPasswd]
#
# Удаление пользователя
# ^oPass.delete[$sUserName]
@CLASS
htpasswd

##############################
# Конструктор
# Параметр:
# sFileName - полный путь к файлу .htpasswd (вида /path/.htpasswd)
# Поля класса:
# _sFileName - полный путь к файлу .htpasswd (вида /path/.htpasswd)
# _hPasswd - хэш пользователь/пароль
@init[sFileName][lHtpass]
$_sFileName[$sFileName]
^try{
	$lHtpass[^file::load[text;$sFileName]]
	$lHtpass[^lHtpass.text.match[\n][g]{^#0A}]
	$lHtpass[^table::create{login	password^#0A^lHtpass.match[:][g]{	}}]
}{
	^if($exception.type eq file.missing
		|| $exception.type eq parser.runtime
	){
		$exception.handled(1)
		$lHtpass[^table::create{login	password}]
	}
}
$_hPasswd[^lHtpass.hash[login]]

##############################
# Добавление пользователя / изменение пароля
@setpw[sLogin;sPassword][lCrypted]
$lCrypted[^math:crypt[$sPassword;^$apr1^$]]
^setpwCrypted[$sLogin;$lCrypted]

##############################
# Добавление пользователя / изменение пароля
# с передачей не самого пароля а уже его хэша
@setpwCrypted[sLogin;sPassword]
^if(def $sLogin && !^sLogin.match[:]){
	^_hPasswd.add[
		$.[$sLogin][
			$.login[$sLogin]
			$.password[$sPassword]
		]
	]
	^_save[]
}

##############################
# Удаление пользователя
@delete[sLogin]
^if(def $_hPasswd.[$sLogin]){
	^_hPasswd.delete[$sLogin]
	^_save[]
}

##############################
# Сохранение .htpasswd
@_save[]
$result[^_hPasswd.foreach[login;password]{${login}:${password.password}}[^#0A]]
^try{
	^result.save[$_sFileName]
	$result[]
}{
	$exception.handled(1)
	$result[Невозможно записать $_sFileName]
}